MAX_QUESTWATCH_LINES = 30;
MAX_WATCHABLE_QUESTS = 15;
MAX_QUEST_WATCH_TIME = 86400;
QUEST_WATCH_NO_EXPIRE = 1;

QuestGuru_WatchList = {};
QuestGuru_WatchLines = 0;

function QuestGuru_SetWatchBorder(index)
	if (index == 1) then
		QuestGuru_QuestWatchFrame:SetBackdrop({ bgFile="", edgeFile="" });
	elseif (index == 2) then
		QuestGuru_QuestWatchFrame:SetBackdrop({
			bgFile="Interface\\Characterframe\\UI-Party-Background",
			edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
			tile=1, tileSize=16, edgeSize=16,
			insets={left=5, right=5, top=5, bottom=5}
		});
	end
	QuestGuru_QuestWatchFrame:SetBackdropColor(0,0,0,QuestGuru_Settings.Tracker.Alpha);
end

do
	local temp, i;

	QuestGuru_QuestWatchFrame = CreateFrame("FRAME", "QuestGuru_QuestWatchFrame", UIParent);
	QuestGuru_QuestWatchFrame:Hide();
	QuestGuru_QuestWatchFrame:EnableMouse();
	QuestGuru_QuestWatchFrame:SetMovable(1);
	QuestGuru_QuestWatchFrame:SetResizable(1);
	QuestGuru_QuestWatchFrame:SetToplevel(1);
	QuestGuru_QuestWatchFrame:SetClampedToScreen(true);
	QuestGuru_QuestWatchFrame:SetWidth(300);
	QuestGuru_QuestWatchFrame:SetHeight(20);
	QuestGuru_QuestWatchFrame:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOMRIGHT", 0, 10);
	QuestGuru_SetWatchBorder(2);
	
	QuestGuru_QuestWatchFrame:RegisterForDrag("LeftButton");
	QuestGuru_QuestWatchFrame:SetScript("OnDragStart",
		function ()
			this:StartMoving();
			this.isMoving = true;
 		end);
 	QuestGuru_QuestWatchFrame:SetScript("OnDragStop",
 		function ()
			this:StopMovingOrSizing();
			this.isMoving = false;
			QuestGuru_Settings.Tracker.Left = QuestGuru_QuestWatchFrame:GetLeft();
		    QuestGuru_Settings.Tracker.Top = QuestGuru_QuestWatchFrame:GetTop();
		end);

	temp = QuestGuru_QuestWatchFrame:CreateFontString("QuestGuru_QuestWatchQuestName", "BACKGROUND", "GameFontNormal");
	temp:SetPoint("TOPLEFT", 4, 0);
	temp:SetJustifyH("LEFT");
	temp:SetWidth(0);
	temp:SetHeight(12);
--	_G["QuestWatchQuestName"] = temp;

	for i=1, MAX_QUESTWATCH_LINES do
		temp = QuestGuru_QuestWatchFrame:CreateFontString("QuestGuru_QuestWatchLine"..i, "BACKGROUND", "GameFontHighlight");
		temp:SetPoint("TOPLEFT", QuestGuru_QuestWatchFrame, 8, -7);
		temp:SetWidth(0);
		temp:SetHeight(12);
--		_G["QuestWatchLine"..i] = temp;
	end
	
--	QuestWatchFrame = QuestGuru_QuestWatchFrame;
end

-- QuestWatch functions
local old_QuestWatch_Update = QuestWatch_Update;
function QuestWatch_Update()
	if (not QuestGuru_Settings.Tracker.Enabled) then
	    QuestGuru_QuestWatchFrame:Hide();
	    old_QuestWatch_Update();
	    return;
	end
	QuestWatchFrame:Hide();
	local numObjectives;
	local questWatchMaxWidth = 0;
	local tempWidth;
	local watchText;
	local text, type, finished;
	local questTitle
	local watchTextIndex = 1;
	local numHeaders = 0;
	local numTitles=0;
	local numObjs=0;
	local questIndex;
	local objectivesCompleted;
	local strLvl;
	local currHeader = "";
	local lastWatchHeader = "";

	local numEntries, numQuests = GetNumQuestLogEntries();

	for questIndex=1, numEntries do
		questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
		if (isHeader) then
		    currHeader = questLogTitleText;
		end
		if (IsQuestWatched(questIndex)) then
			if (currHeader ~= lastWatchHeader and QuestGuru_Settings.Tracker.ShowHeaders) then
				watchText = getglobal("QuestGuru_QuestWatchLine"..watchTextIndex);
				if ( watchTextIndex > 1 ) then
					watchText:SetPoint("TOPLEFT", "QuestGuru_QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, -6);
				end
				watchText:SetText(currHeader);
				watchText:SetTextColor(1, 1, 1);
				lastWatchHeader = currHeader;
				watchText:Show();
		      	watchTextIndex = watchTextIndex + 1;
		      	numHeaders = numHeaders + 1;
			end
			
			numObjectives = GetNumQuestLeaderBoards(questIndex);
		
			--If there are objectives set the title
			if ( numObjectives > 0 ) then
				-- Set title
				watchText = getglobal("QuestGuru_QuestWatchLine"..watchTextIndex);
				strLvl = "["..level
				if (questTag ~= nil) then
					strLvl = strLvl.."+"
				end
				strLvl = strLvl.."] "
				watchText:SetText(strLvl..GetQuestLogTitle(questIndex));
				tempWidth = watchText:GetWidth();
				-- Set the anchor of the title line a little lower
				if ( watchTextIndex > 1 ) then
					watchText:SetPoint("TOPLEFT", "QuestGuru_QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, -4);
				end
				watchText:Show();
				if ( tempWidth > questWatchMaxWidth ) then
					questWatchMaxWidth = tempWidth;
				end
				watchTextIndex = watchTextIndex + 1;
				numTitles = numTitles + 1;
				objectivesCompleted = 0;
				for j=1, numObjectives do
					text, type, finished = GetQuestLogLeaderBoard(j, questIndex);
					watchText = getglobal("QuestGuru_QuestWatchLine"..watchTextIndex);
					-- Set Objective text
					watchText:SetText(" - "..text);
					-- Color the objectives
					if ( finished ) then
						watchText:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
						objectivesCompleted = objectivesCompleted + 1;
					else
						watchText:SetTextColor(0.8, 0.8, 0.8);
					end
					tempWidth = watchText:GetWidth();
					if ( tempWidth > questWatchMaxWidth ) then
						questWatchMaxWidth = tempWidth;
					end
					watchText:SetPoint("TOPLEFT", "QuestGuru_QuestWatchLine"..(watchTextIndex - 1), "BOTTOMLEFT", 0, 0);
					watchText:Show();
					watchTextIndex = watchTextIndex + 1;
					numObjs = numObjs + 1;
				end
				-- Brighten the quest title if all the quest objectives were met
				watchText = getglobal("QuestGuru_QuestWatchLine"..watchTextIndex-numObjectives-1);
				color = GetDifficultyColor(level);
				if ( objectivesCompleted == numObjectives ) then
					watchText:SetTextColor(color.r, color.g, color.b);
				else
					watchText:SetTextColor(color.r * 0.75, color.g * 0.75, color.b * 0.75);
				end
			end
		end
	end

	-- Set tracking indicator
	if ( GetNumQuestWatches() > 0 ) then
		QuestGuru_QuestLogTrackTracking:SetVertexColor(0, 1.0, 0);
	else
		QuestGuru_QuestLogTrackTracking:SetVertexColor(1.0, 0, 0);
	end
	
	-- If no watch lines used then hide the frame and return
	if ( watchTextIndex == 1 ) then
		QuestGuru_QuestWatchFrame:Hide();
		return;
	else
		QuestGuru_QuestWatchFrame:Show();
		QuestGuru_QuestWatchFrame:SetHeight(((numHeaders-1)*6)+(numTitles*4)+(watchTextIndex*12)+8);
		QuestGuru_QuestWatchFrame:SetWidth(questWatchMaxWidth + 25);
		if (QuestGuru_Settings.Tracker.Left == nil) then
			QuestGuru_Settings.Tracker.Left = MinimapCluster:GetLeft();
		end
		if (QuestGuru_Settings.Tracker.Top == nil) then
		    QuestGuru_Settings.Tracker.Top = MinimapCluster:GetBottom()+10;
		end
		QuestGuru_QuestWatchFrame:ClearAllPoints();
		QuestGuru_QuestWatchFrame:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", QuestGuru_Settings.Tracker.Left, QuestGuru_Settings.Tracker.Top);
	end

	QuestGuru_WatchLines = watchTextIndex;
	
	-- Hide unused watch lines
	for i=watchTextIndex, MAX_QUESTWATCH_LINES do
		getglobal("QuestGuru_QuestWatchLine"..i):Hide();
	end

	UIParent_ManageFramePositions();
end

local old_AddQuestWatch = AddQuestWatch;
function AddQuestWatch(questIndex, ...)
	if (not QuestGuru_Settings.Tracker.Enabled) then
	    QuestGuru_WatchList = {};
	    old_AddQuestWatch(questIndex);
	    return;
	end
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
	if (QuestGuru_WatchList[questLogTitleText] == nil) then
	    QuestGuru_WatchList[questLogTitleText] = {};
	end
	if (QuestGuru_WatchList[questLogTitleText].Watched) then
	    return;
	end
	if (GetNumQuestLeaderBoards(questIndex) + QuestGuru_WatchLines + 2 > MAX_QUESTWATCH_LINES) then -- +2 for header and title
		UIErrorsFrame:AddMessage("The QuestGuru quest tracker is currently full", 1.0, 0.1, 0.1, 1.0);
	    return;
	end
	QuestGuru_WatchList[questLogTitleText].Watched = true;
end

local old_RemoveQuestWatch = RemoveQuestWatch;
function RemoveQuestWatch(questIndex, ...)
	if (not QuestGuru_Settings.Tracker.Enabled) then
	    QuestGuru_WatchList = {};
	    old_RemoveQuestWatch(questIndex);
	    return;
	end
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
	QuestGuru_WatchList[questLogTitleText] = nil;
end

local old_IsQuestWatched = IsQuestWatched;
function IsQuestWatched(questIndex, ...)
	if (not QuestGuru_Settings.Tracker.Enabled) then
	    return old_IsQuestWatched(questIndex);
	end
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
	if (QuestGuru_WatchList[questLogTitleText] == nil) then
		return false;
	end
	return QuestGuru_WatchList[questLogTitleText].Watched;
end
